@@ -1,12 +1,12 @@
 /************************************************************************
  *	Whatever is needed for (un)locking files in various ways	*
  *									*
- *	Copyright (c) 1990-1997, S.R. van den Berg, The Netherlands	*
+ *	Copyright (c) 1990-1999, S.R. van den Berg, The Netherlands	*
  *	#include "../README"						*
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: locking.c,v 1.47 1997/04/03 01:58:44 srb Exp $";
+ "$Id: locking.c,v 1.52 1999/03/30 06:17:18 guenther Exp $";
 #endif
 #include "procmail.h"
 #include "robust.h"
@@ -22,23 +22,24 @@ void lockit(name,lockp)char*name;char**const lockp;
   zombiecollect();
   if(*lockp)
    { if(!strcmp(name,*lockp))	/* compare the previous lockfile to this one */
-	return;			 /* they're equal, save yourself some effort */
+      { free(name);return;	 /* they're equal, save yourself some effort */
+      }
      unlock(lockp);		       /* unlock any previous lockfile FIRST */
    }				  /* to prevent deadlocks (I hate deadlocks) */
   if(!*name)
-     return;
+   { free(name);return;
+   }
   if(!strcmp(name,defdeflock))	       /* is it the system mailbox lockfile? */
    { locktype=doCHECK|doLOCK;
+     if(sgid!=gid&&setegid(sgid))      /* try and get some extra permissions */
 #ifndef fdlock
-     if(!accspooldir)
-      { yell("Bypassed locking",name);
-	return;
-      }
-     else
+	if(!accspooldir)
+	 { yell("Bypassed locking",name);
+	   free(name);return;
+	 }
 #endif
-	setegid(sgid);		       /* try and get some extra permissions */
+	;
    }
-  name=tstrdup(name); /* allocate now, so we won't hang on memory *and* lock */
   for(lcking|=lck_LOCKFILE;;)
    { yell("Locking",name);	    /* in order to cater for clock skew: get */
      if(!xcreat(name,LOCKperm,&t,locktype))    /* time t from the filesystem */
@@ -103,9 +104,16 @@ term: { free(name);			     /* drop the preallocated buffer */
 void lcllock P((void))				    /* lock a local lockfile */
 { char*lckfile;			    /* locking /dev/null or | would be silly */
   if(tolock||strcmp(buf2,devnull)&&strcmp(buf2,"|"))
-   { lckfile=tolock?tolock:strcat(buf2,lockext);
+   { if(tolock)
+	lckfile=tstrdup(tolock);
+     else
+      { int len=strlen(buf2);
+	strcpy(strcpy(lckfile=malloc(len+strlen(lockext)+1),buf2)+len,lockext);
+      }
      if(globlock&&!strcmp(lckfile,globlock))	 /* same as global lockfile? */
-	nlog("Deadlock attempted on"),logqnl(lckfile);
+      { nlog("Deadlock attempted on");logqnl(lckfile);
+	free(lckfile);
+      }
      else
 	lockit(lckfile,&loclock);
    }
@@ -231,6 +239,7 @@ ufcntl:
      oldfdlock=fd;lcking&=~lck_KERNEL;
      return ret;
    }
+  return 1;							/* timed out */
 }
 
 int fdunlock P((void))
