@@ -1,12 +1,12 @@
 /************************************************************************
  *	Routines that deal with the mailfolder(format)			*
  *									*
- *	Copyright (c) 1990-1997, S.R. van den Berg, The Netherlands	*
+ *	Copyright (c) 1990-1999, S.R. van den Berg, The Netherlands	*
  *	#include "../README"						*
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: mailfold.c,v 1.70 1997/04/03 01:58:45 srb Exp $";
+ "$Id: mailfold.c,v 1.76 1999/02/21 19:37:15 guenther Exp $";
 #endif
 #include "procmail.h"
 #include "acommon.h"
@@ -68,7 +68,8 @@ long dump(s,source,len)const int s;const char*source;long len;
 	if(part&&tofile==to_FILE)			    /* a_time<m_time */
 	 { struct stat stbuf;
 	   rwrite(s,source++,1);len--;part--;		     /* set the trap */
-	   if(fstat(s,&stbuf)||stbuf.st_mtime==stbuf.st_atime)	  /* needed? */
+	   if(fstat(s,&stbuf)||					  /* needed? */
+	    stbuf.st_mtime==stbuf.st_atime&&stbuf.st_size!=1)
 	      ssleep(1);  /* ...what a difference this (tea) second makes... */
 	 }
 #endif
@@ -115,9 +116,13 @@ static int dirfile(chp,linkonly)char*const chp;const int linkonly;
      else
 	readerr(buf);
 #endif /* NOopendir */
+     if(chp-buf+sizeNUM(i)-XTRAlinebuf>linebuf)
+exlb: { nlog(exceededlb);setoverflow();
+	goto ret;
+      }
      ;{ int ok;
 	do ultstr(0,++i,chp);		       /* find first empty MH folder */
-	while((ok=linkonly?link(buf2,buf):hlink(buf2,buf))&&errno==EEXIST);
+	while((ok=linkonly?rlink(buf2,buf,0):hlink(buf2,buf))&&errno==EEXIST);
 	if(linkonly)
 	 { yell(lkingto,buf);
 	   if(ok)
@@ -129,13 +134,16 @@ static int dirfile(chp,linkonly)char*const chp;const int linkonly;
      goto opn;
    }
   ;{ struct stat stbuf;
+     char*p=strchr(buf,'\0');
+     if(p-buf+strlen(msgprefix)+sizeNUM(stbuf.st_ino)-XTRAlinebuf>linebuf)
+	goto exlb;
      stat(buf2,&stbuf);
      ultoan((unsigned long)stbuf.st_ino,      /* filename with i-node number */
-      strchr(strcat(buf,msgprefix),'\0'));
+      strchr(strcat(p,msgprefix),'\0'));
    }
   if(linkonly)
    { yell(lkingto,buf);
-     if(link(buf2,buf))	   /* hardlink the new file, it's a directory folder */
+     if(rlink(buf2,buf,0)) /* hardlink the new file, it's a directory folder */
 nolnk:	nlog("Couldn't make link to"),logqnl(buf);
      else
 didlnk:
@@ -197,6 +205,10 @@ makefile:
      *chp='\0',chp[-1]= *MCDIRSEP_,strcpy(buf2,buf);	   /* it ended in /. */
   else					 /* fixup directory name, append a / */
      strcat(chp,MCDIRSEP_),strcpy(buf2,buf),chp=0;
+  if(strlen(buf2)+UNIQnamelen>linebuf)
+   { nlog(exceededlb);setoverflow();
+     return -1;
+   }
   ;{ int fd= -1;		/* generate the name for the first directory */
      if(unique(buf2,strchr(buf2,'\0'),NORMperm,verbose,0)&&
       (fd=dirfile(chp,0))>=0&&linkfolder)	 /* save the file descriptor */
