@@ -1,12 +1,12 @@
 /************************************************************************
  *	Miscellaneous routines used by procmail				*
  *									*
- *	Copyright (c) 1990-1995, S.R. van den Berg, The Netherlands	*
+ *	Copyright (c) 1990-1999, S.R. van den Berg, The Netherlands	*
  *	#include "../README"						*
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: misc.c,v 1.71 1995/10/30 02:09:24 srb Exp $";
+ "$Id: misc.c,v 1.89.2.1 1999/04/06 04:05:16 guenther Exp $";
 #endif
 #include "procmail.h"
 #include "acommon.h"
@@ -21,11 +21,13 @@ static /*const*/char rcsid[]=
 #include "mcommon.h"
 #include "goodies.h"
 #include "locking.h"
+#include "../patchlevel.h"
 #ifndef NO_COMSAT
 #include "network.h"
 #endif
 #include "mailfold.h"
 #include "lastdirsep.h"
+#include "authenticate.h"
 
 struct varval strenvvar[]={{"LOCKSLEEP",DEFlocksleep},
  {"LOCKTIMEOUT",DEFlocktimeout},{"SUSPEND",DEFsuspend},
@@ -34,12 +36,12 @@ struct varval strenvvar[]={{"LOCKSLEEP",DEFlocksleep},
 struct varstr strenstr[]={{"SHELLMETAS",DEFshellmetas},{"LOCKEXT",DEFlockext},
  {"MSGPREFIX",DEFmsgprefix},{"COMSAT",""},{"TRAP",""},
  {"SHELLFLAGS",DEFshellflags},{"DEFAULT",DEFdefault},{"SENDMAIL",DEFsendmail},
- {"SENDMAILFLAGS",DEFflagsendmail}};
+ {"SENDMAILFLAGS",DEFflagsendmail},{"PROCMAIL_VERSION",PM_VERSION}};
 
 #define MAXvarvals	 maxindex(strenvvar)
 #define MAXvarstrs	 maxindex(strenstr)
 
-const char lastfolder[]="LASTFOLDER",maildir[]="MAILDIR";
+const char lastfolder[]="LASTFOLDER",maildir[]="MAILDIR",slinebuf[]="LINEBUF";
 int didchd;
 char*globlock;
 static time_t oldtime;
@@ -171,7 +173,11 @@ int nextrcfile P((void))	/* next rcfile specified on the command line */
   while(p= *gargv)
    { gargv++;
      if(!strchr(p,'='))
-      { rcfile=p;
+      { if(strlen(p)>linebuf-1)
+	 { nlog("Excessively long rcfile path skipped\n");
+	   continue;
+	 }
+	rcfile=p;
 	return rval;
       }
      rval=1;			       /* not the first argument encountered */
@@ -210,7 +216,8 @@ void sterminate P((void))
 }
 
 void Terminate P((void))
-{ ignoreterm();
+{ const char*chp;
+  ignoreterm();
   if(retvl2!=EXIT_SUCCESS)
      fakedelivery=0,retval=retvl2;
   if(getpid()==thepid)
@@ -224,7 +231,9 @@ void Terminate P((void))
 	lstfolder=tgetenv(lastfolder);
      logabstract(lstfolder);
 #ifndef NO_COMSAT
-     ;{ int s;struct sockaddr_in addr;char*chp,*chad;	     /* @ seperator? */
+     if(strlen(chp=tgetenv(lgname))+2<=linebuf)	  /* first pass length check */
+      { int s;struct sockaddr_in addr;char*chad;	     /* @ seperator? */
+	cat(chp,"@");			     /* start setting up the message */
 	if(chad=strchr(chp=(char*)scomsat,SERV_ADDRsep))
 	   *chad++='\0';	      /* split it up in service and hostname */
 	else if(!renvint(-1L,chp))		/* or is it a false boolean? */
@@ -263,13 +272,12 @@ void Terminate P((void))
 	 }
 	else
 	   addr.sin_port=htons((short)s);		    /* network order */
-	cat(tgetenv(lgname),"@");		 /* should always fit in buf */
 	if(lasttell>=0)					   /* was it a file? */
 	   ultstr(0,(unsigned long)lasttell,buf2),catlim(buf2);	      /* yep */
 	catlim(COMSATxtrsep);				 /* custom seperator */
 	if(lasttell>=0&&!strchr(dirsep,*lstfolder))    /* relative filename? */
 	   catlim(tgetenv(maildir)),catlim(MCDIRSEP_);	   /* prepend curdir */
-	catlim(lstfolder);s=socket(AF_INET,SOCK_DGRAM,UDP_protocolno);
+	catlim(lstfolder);s=socket(PF_INET,SOCK_DGRAM,UDP_protocolno);
 	sendto(s,buf,strlen(buf),0,(const void*)&addr,sizeof(addr));rclose(s);
 	yell("Notified comsat:",buf);
       }
@@ -301,7 +309,9 @@ int alphanum(c)const unsigned c;
 }
 
 char*pmrc2buf P((void))
-{ sgetcp=pmrc;readparse(buf,sgetc,2);
+{ sgetcp=pmrc;
+  if(readparse(buf,sgetc,2))
+     buf[0]='\0';
   return buf;
 }
 
@@ -311,6 +321,10 @@ void setmaildir(newdir)const char*const newdir;		    /* destroys buf2 */
   strcpy(++chp,newdir);sputenv(buf2);
 }
 
+void setoverflow P((void))
+{ sputenv("PROCMAIL_OVERFLOW=yes");
+}
+
 void srequeue P((void))
 { retval=EX_TEMPFAIL;sterminate();
 }
@@ -335,23 +349,23 @@ void catlim(src)register const char*src;
 
 void setdef(name,contents)const char*const name,*const contents;
 { strcat(strcat(strcpy((char*)(sgetcp=buf2),name),"="),contents);
-  readparse(buf,sgetc,2);sputenv(buf);
+  if(!readparse(buf,sgetc,2))
+     sputenv(buf);
 }
 
 void metaparse(p)const char*p;				    /* result in buf */
 { if(sh=!!strpbrk(p,shellmetas))
      strcpy(buf,p);			 /* copy literally, shell will parse */
   else
-#ifndef GOT_bin_test
    { sgetcp=p=tstrdup(p);
-     readparse(buf,sgetc,0);				/* parse it yourself */
-     if(!strcmp(test,buf))
-	strcpy(buf,p),sh=1;			       /* oops, `test' found */
+     if(readparse(buf,sgetc,0)				/* parse it yourself */
+#ifndef GOT_bin_test
+	||!strcmp(test,buf)
+#endif
+	)
+	strcpy(buf,p),sh=1;		   /* oops, overflow or `test' found */
      free((char*)p);
    }
-#else
-     sgetcp=p,readparse(buf,sgetc,0);
-#endif
 }
 
 void concatenate(p)register char*p;
@@ -393,12 +407,16 @@ void setlastfolder(folder)const char*const folder;
    }
 }
 
-char*gobenv(chp)char*chp;
+char*gobenv(chp,end)char*chp,*end;
 { int found,i;
-  found=0;
+  found=0;end--;
   if(alphanum(i=getb())&&!numeric(i))
-     for(found=1;*chp++=i,alphanum(i=getb()););
+     for(found=1;*chp++=i,chp<end&&alphanum(i=getb()););
   *chp='\0';ungetb(i);
+  if(chp==end)							 /* overflow */
+   { nlog(exceededlb);setoverflow();
+     return end+1;
+   }
   switch(i)
    { case ' ':case '\t':case '\n':case '=':
 	if(found)
@@ -414,22 +432,33 @@ int asenvcpy(src)char*src;
      *	really change the uid now, since it would not be safe to
      *	evaluate the extra command line arguments otherwise
      */
-   { setids();strcpy(buf,src);src=buf+(chp-src);
-     strcpy((char*)(sgetcp=buf2),++src);readparse(src,sgetc,2);
-     chp=sputenv(buf);src[-1]='\0';asenv(chp);
+   { erestrict=1;setids();strcpy(buf,src);src=buf+(chp-src);
+     strcpy((char*)(sgetcp=buf2),++src);
+     if(!readparse(src,sgetc,2))
+      { chp=sputenv(buf);src[-1]='\0';
+	asenv(chp);
+      }
      return 1;
    }
   return 0;
 }
 
+void mallocbuffers(linebuf)size_t linebuf;
+{ if(buf)
+   { free(buf);
+     free(buf2);
+   }
+  buf=malloc(linebuf);buf2=malloc(linebuf);
+}
+
 void asenv(chp)const char*const chp;
-{ static const char slinebuf[]="LINEBUF",logfile[]="LOGFILE",Log[]="LOG",
-   sdelivered[]="DELIVERED",includerc[]="INCLUDERC",eumask[]="UMASK",
-   dropprivs[]="DROPPRIVS",shift[]="SHIFT";
+{ static const char logfile[]="LOGFILE",Log[]="LOG",sdelivered[]="DELIVERED",
+   includerc[]="INCLUDERC",eumask[]="UMASK",dropprivs[]="DROPPRIVS",
+   shift[]="SHIFT";
   if(!strcmp(buf,slinebuf))
    { if((linebuf=renvint(0L,chp)+XTRAlinebuf)<MINlinebuf+XTRAlinebuf)
 	linebuf=MINlinebuf+XTRAlinebuf;		       /* check minimum size */
-     free(buf);free(buf2);buf=malloc(linebuf);buf2=malloc(linebuf);
+     mallocbuffers(linebuf);
    }
   else if(!strcmp(buf,maildir))
      if(chdir(chp))
@@ -468,7 +497,7 @@ void asenv(chp)const char*const chp;
       }
    }
   else if(!strcmp(buf,lockfile))
-     lockit((char*)chp,&globlock);
+     lockit(tstrdup((char*)chp),&globlock);
   else if(!strcmp(buf,eumask))
      doumask((mode_t)strtol(chp,(char**)0,8));
   else if(!strcmp(buf,includerc))
@@ -555,24 +584,11 @@ char*egrepin(expr,source,len,casesens)char*expr;const char*source;
   return (char*)source;
 }
 
-const struct passwd*savepass(spass,uid)struct passwd*const spass;
- const uid_t uid;
-{ struct passwd*tpass;
-  if(spass->pw_name&&spass->pw_uid==uid)
-     goto ret;
-  if(tpass=getpwuid(uid))				  /* save by copying */
-   { spass->pw_uid=tpass->pw_uid;spass->pw_gid=tpass->pw_gid;
-     spass->pw_name=cstr(spass->pw_name,tpass->pw_name);
-     spass->pw_dir=cstr(spass->pw_dir,tpass->pw_dir);
-     spass->pw_shell=cstr(spass->pw_shell,tpass->pw_shell);
-ret: return spass;
-   }
-  return (const struct passwd*)0;
-}
-
-int enoughprivs(passinvk,euid,egid,uid,gid)const struct passwd*const passinvk;
+int enoughprivs(passinvk,euid,egid,uid,gid)const auth_identity*const passinvk;
  const uid_t euid,uid;const gid_t egid,gid;
-{ return euid==ROOT_uid||passinvk&&passinvk->pw_uid==uid||euid==uid&&egid==gid;
+{ return euid==ROOT_uid||
+   passinvk&&auth_whatuid(passinvk)==uid||
+   euid==uid&&egid==gid;
 }
 
 void initdefenv P((void))
@@ -595,6 +611,13 @@ void rcst_nosgid P((void))
 { if(!rcstate)
      rcstate=rc_NOSGID;
 }
+
+static void inodename(stbuf,i)const struct stat*const stbuf;const size_t i;
+{ static const char bogusprefix[]=BOGUSprefix;char*p;
+  p=strchr(strcpy(strcpy(buf+i,bogusprefix)+STRLEN(bogusprefix),
+   getenv(lgname)),'\0');
+  *p++='.';ultoan((unsigned long)stbuf->st_ino,p);	  /* i-node numbered */
+}
 			     /* lifted out of main() to reduce main()'s size */
 int screenmailbox(chp,chp2,egid,Deliverymode)
  char*chp;char*const chp2;const gid_t egid;const int Deliverymode;
@@ -626,7 +649,9 @@ keepgid:			   /* keep the gid from the parent directory */
 	   setgid(sgid);     /* we were started nosgid, but we might need it */
    }
   else				/* panic, mail-spool directory not available */
-     setids(),mkdir(buf,NORMdirperm);	     /* try creating the last member */
+   { int c;				     /* try creating the last member */
+     setids();c=buf[i-1];buf[i-1]='\0';mkdir(buf,NORMdirperm);buf[i-1]=c;
+   }
  /*
   *	  check if the default-mailbox-lockfile is owned by the
   *	  recipient, if not, mark it for further investigation, it
@@ -638,8 +663,7 @@ keepgid:			   /* keep the gid from the parent directory */
 	 renfbogus[]="Couldn't rename bogus \"%s\" into \"%s\"";
 	;{ int goodlock;
 	   if(!(goodlock=lstat(defdeflock,&stbuf)||stbuf.st_uid==uid))
-	      ultoan((unsigned long)stbuf.st_ino,	  /* i-node numbered */
-	       strchr(strcpy(buf+i,BOGUSprefix),'\0'));
+	      inodename(&stbuf,i);
 	  /*
 	   *	  check if the original/default mailbox of the recipient
 	   *	  exists, if it does, perform some security checks on it
@@ -664,26 +688,38 @@ boglock:	 if(!goodlock)		      /* try & rename bogus lockfile */
 	    }
 	 }
 	if(mboxstat>0||mboxstat<0&&(setids(),!lstat(chp,&stbuf)))
-	   if(!(stbuf.st_mode&S_IWUSR)||	     /* recipient can write? */
-	      S_ISLNK(stbuf.st_mode)||			/* no symbolic links */
-	      (S_ISDIR(stbuf.st_mode)?	      /* directories, yes, hardlinks */
-		!(stbuf.st_mode&S_IXUSR):stbuf.st_nlink!=1))	       /* no */
-	      goto bogusbox;		/* can't deliver to this contraption */
-	   else if(stbuf.st_uid!=uid)		      /* recipient not owner */
-bogusbox:   { ultoan((unsigned long)stbuf.st_ino,	  /* i-node numbered */
-	       strchr(strcpy(buf+i,BOGUSprefix),'\0'));		    /* bogus */
-	      nlog("Renaming bogus mailbox \"");elog(chp);elog("\" into");
-	      logqnl(buf);
-	      if(rename(chp,buf))	   /* try and move it out of the way */
-	       { syslog(LOG_EMERG,renfbogus,chp,buf);
-		 goto fishy;	    /* rename failed, something's fishy here */
-	       }
+	 { int checkiter=1;
+	   for(;;)
+	    { if(stbuf.st_uid!=uid||		      /* recipient not owner */
+		 !(stbuf.st_mode&S_IWUSR)||	     /* recipient can write? */
+		 S_ISLNK(stbuf.st_mode)||		/* no symbolic links */
+		 (S_ISDIR(stbuf.st_mode)?     /* directories, yes, hardlinks */
+		   !(stbuf.st_mode&S_IXUSR):stbuf.st_nlink!=1))	       /* no */
+		/*
+		 *	If another procmail is about to create the new
+		 *	mailbox, and has just made the link, st_nlink==2
+		 */
+		 if(checkiter--)	    /* maybe it was a race condition */
+		    suspend();		 /* close eyes, and hope it improves */
+		 else			/* can't deliver to this contraption */
+		  { inodename(&stbuf,i);nlog("Renaming bogus mailbox \"");
+		    elog(chp);elog("\" into");logqnl(buf);
+		    if(rename(chp,buf))	   /* try and move it out of the way */
+		     { syslog(LOG_EMERG,renfbogus,chp,buf);
+		       goto fishy;  /* rename failed, something's fishy here */
+		     }
+		    else
+		       syslog(LOG_ALERT,renbogus,chp,buf);
+		    goto nobox;
+		  }
 	      else
-		 syslog(LOG_ALERT,renbogus,chp,buf);
+		 break;
+	      if(lstat(chp,&stbuf))
+		 goto nobox;
 	    }					/* SysV type autoforwarding? */
-	   else if(Deliverymode&&stbuf.st_mode&(S_ISGID|S_ISUID))
+	   if(Deliverymode&&stbuf.st_mode&(S_ISGID|S_ISUID))
 	    { nlog("Autoforwarding mailbox found\n");
-	      return EX_NOUSER;
+	      exit(EX_NOUSER);
 	    }
 	   else
 	    { if(!(stbuf.st_mode&OVERRIDE_MASK)&&
@@ -697,7 +733,9 @@ bogusbox:   { ultoan((unsigned long)stbuf.st_ino,	  /* i-node numbered */
 	       }
 	      break;				  /* everything is just fine */
 	    }
+	 }
       }
+nobox:
      if(!(accspooldir&1))	     /* recipient does not own the spool dir */
       { if(!xcreat(chp,NORMperm,(time_t*)0,doCHOWN|doCHECK))	   /* create */
 	   break;		   /* mailbox... yes we could, fine, proceed */
@@ -718,8 +756,8 @@ fishy:
 			     /* lifted out of main() to reduce main()'s size */
 int conditions(flags,prevcond,lastsucc,lastcond,nrcond)const char flags[];
  const int prevcond,lastsucc,lastcond;int nrcond;
-{ char*chp,*chp2,*startchar;double score;int scored,i;long tobesent;
-  static const char suppressed[]=" suppressed\n";
+{ char*chp,*chp2,*startchar;double score;int scored,i,skippedempty;
+  long tobesent;static const char suppressed[]=" suppressed\n";
   score=scored=0;
   if(flags[ERROR_DO]&&flags[ELSE_DO])
      nlog(conflicting),elog("else-if-flag"),elog(suppressed);
@@ -749,7 +787,7 @@ noconcat:
    if(flags[ALSO_NEXT_RECIPE])
       i=i&&lastcond;
    Stdout=0;
-   for(;;)
+   for(skippedempty=0;;)
     { skipspace();--nrcond;
       if(!testB('*'))			    /* marks a condition, new syntax */
 	 if(nrcond<0)		/* keep counting, for backward compatibility */
@@ -758,10 +796,18 @@ noconcat:
 	       continue;
 	     }
 	    if(testB('\n'))				 /* skip empty lines */
+	     { skippedempty=1;			 /* make a note of this fact */
 	       continue;
+	     }
+	    if(skippedempty&&testB(':'))
+	     { nlog("Missing action\n");i=2;
+	       goto ret;
+	     }
 	    break;		     /* no more conditions, time for action! */
 	  }
-      skipspace();getlline(buf2);
+      skipspace();
+      if(getlline(buf2))
+	 i=0;				       /* assume failure on overflow */
       if(i)					 /* check out all conditions */
        { int negate,scoreany;double weight,xponent,lscore;
 	 char*lstartchar=startchar;long ltobesent=tobesent,sizecheck=filled;
@@ -852,8 +898,7 @@ jinregs:		regsp=regs;		/* start over and look again */
 		     igncase=or_nocase||!flags[DISTINGUISH_CASE];
 		     if(scoreany)
 		      { struct eps*re;
-			re=bregcomp(chp,igncase);
-			chp=lstartchar;
+			re=bregcomp(chp,igncase);chp=lstartchar;
 			if(negate)
 			 { if(weight&&!bregexec(re,(const uchar*)chp,
 			    (const uchar*)chp,(size_t)ltobesent,igncase))
@@ -876,8 +921,8 @@ jinregs:		regsp=regs;		/* start over and look again */
 				    score+=weight<0?MIN32:MAX32;
 				 break;			    /* matches early */
 			       }
-			      ;{ double nweight;
-				 if((nweight=weight*weight)<oweight&&oweight<1)
+			      ;{ volatile double nweight=weight*weight;
+				 if(nweight<oweight&&oweight<1)
 				    break;
 				 oweight=nweight;
 			       }
@@ -891,7 +936,9 @@ jinregs:		regsp=regs;		/* start over and look again */
 		   }
 		  break;
 		}
-	       case '$':*buf2='"';squeeze(chp);readparse(buf,sgetc,2);
+	       case '$':*buf2='"';squeeze(chp);
+		  if(readparse(buf,sgetc,2)&&(i=0,1))
+		     break;
 		  strcpy(buf2,skpspace(buf));
 		  goto copydone;
 	       case '!':negate^=1;chp2=skpspace(chp);
@@ -911,9 +958,11 @@ copyrest:	  strcpy(buf,chp2);
 		      i=0;
 		   strcpy(buf2,buf);
 		   break;
-	       case '>':case '<':readparse(buf,sgetc,2);
+	       case '>':case '<':
 		{ long pivot;
-		   ;{ char*chp3;
+		  if(readparse(buf,sgetc,2)&&(i=0,1))
+		     break;
+		  ;{ char*chp3;
 		     pivot=strtol(buf+1,&chp3,10);chp=chp3;
 		   }
 		  skipped(skpspace(chp));strcpy(buf2,buf);
@@ -968,5 +1017,6 @@ skiptrue:;
       lastscore=1;					 /* round up +0 to 1 */
    if(scored&&i&&score<=0)
       i=0;					     /* it was not a success */
+ret:
    return i;
 }
