@@ -7,12 +7,12 @@
  *									*
  *	Seems to be perfect.						*
  *									*
- *	Copyright (c) 1990-1995, S.R. van den Berg, The Netherlands	*
+ *	Copyright (c) 1990-1999, S.R. van den Berg, The Netherlands	*
  *	#include "../README"						*
  ************************************************************************/
 #ifdef RCS
 static /*const*/char rcsid[]=
- "$Id: procmail.c,v 1.114 1995/10/30 02:09:25 srb Exp $";
+ "$Id: procmail.c,v 1.137.2.1 1999/04/06 04:05:19 guenther Exp $";
 #endif
 #include "../patchlevel.h"
 #include "procmail.h"
@@ -31,12 +31,14 @@ static /*const*/char rcsid[]=
 #include "locking.h"
 #include "mailfold.h"
 #include "lastdirsep.h"
+#include "authenticate.h"
 
 static const char*const nullp,From_[]=FROM,exflags[]=RECFLAGS,
- drcfile[]="Rcfile:",systm_mbox[]=SYSTEM_MBOX,pmusage[]=PM_USAGE,
- *etcrc=ETCRC,misrecpt[]="Missing recipient\n",extrns[]="Extraneous ",
- ignrd[]=" ignored\n",pardir[]=chPARDIR,curdir[]={chCURDIR,'\0'},
- insufprivs[]="Insufficient privileges\n";
+ drcfile[]="Rcfile:",pmusage[]=PM_USAGE,*etcrc=ETCRC,
+ misrecpt[]="Missing recipient\n",extrns[]="Extraneous ",ignrd[]=" ignored\n",
+ pardir[]=chPARDIR,curdir[]={chCURDIR,'\0'},
+ insufprivs[]="Insufficient privileges\n",
+ attemptst[]="Attempt to fake stamp by";
 char*buf,*buf2,*loclock,*tolock;
 const char shell[]="SHELL",lockfile[]="LOCKFILE",newline[]="\n",binsh[]=BinSh,
  unexpeof[]="Unexpected EOL\n",*const*gargv,*const*restargv= &nullp,*sgetcp,
@@ -44,12 +46,13 @@ const char shell[]="SHELL",lockfile[]="LOCKFILE",newline[]="\n",binsh[]=BinSh,
  lgname[]="LOGNAME",executing[]="Executing",oquote[]=" \"",cquote[]="\"\n",
  procmailn[]="procmail",whilstwfor[]=" whilst waiting for ",home[]="HOME",
  host[]="HOST",*defdeflock,*argv0="",errwwriting[]="Error while writing to",
- slogstr[]="%s \"%s\"",conflicting[]="Conflicting ",orgmail[]="ORGMAIL";
+ slogstr[]="%s \"%s\"",conflicting[]="Conflicting ",orgmail[]="ORGMAIL",
+ exceededlb[]="Exceeded LINEBUF\n",pathtoolong[]=" path too long";
 char*Stdout;
 int retval=EX_CANTCREAT,retvl2=EXIT_SUCCESS,sh,pwait,lcking,rcstate,rc= -1,
  ignwerr,lexitcode=EXIT_SUCCESS,asgnlastf,accspooldir,crestarg,skiprc,
- savstdout,berkeley;
-size_t linebuf=mx(DEFlinebuf+XTRAlinebuf,STRLEN(systm_mbox)<<1);
+ savstdout,berkeley,mailfilter,erestrict;
+size_t linebuf=mx(DEFlinebuf+XTRAlinebuf,1024/*STRLEN(systm_mbox)<<1*/);
 volatile int nextexit;			       /* if termination is imminent */
 pid_t thepid;
 long filled,lastscore;	       /* the length of the mail, and the last score */
@@ -57,6 +60,18 @@ char*themail,*thebody;			    /* the head and body of the mail */
 uid_t uid;
 gid_t gid,sgid;
 
+static auth_identity*savepass(spass,uid)auth_identity*const spass;
+ const uid_t uid;
+{ const auth_identity*tpass;
+  if(auth_filledid(spass)&&auth_whatuid(spass)==uid)
+     goto ret;
+  if(tpass=auth_finduid(uid,0))				  /* save by copying */
+   { auth_copyid(spass,tpass);
+ret: return spass;
+   }
+  return (auth_identity*)0;
+}
+
 #if 0
 #define wipetcrc()	(etcrc&&(etcrc=0,closerc(),1))
 #else
@@ -70,7 +85,7 @@ static int wipetcrc P((void))	  /* stupid function to avoid a compiler bug */
 #endif
 
 main(argc,argv)const char*const argv[];
-{ register char*chp,*chp2;register int i;int suppmunreadable,mailfilter;
+{ register char*chp,*chp2;register int i;int suppmunreadable;
 #if 0				/* enable this if you want to trace procmail */
   kill(getpid(),SIGSTOP);/*raise(SIGSTOP);*/
 #endif
@@ -85,7 +100,7 @@ main(argc,argv)const char*const argv[];
 	for(presenviron=argc=0;(chp2=(char*)argv[++argc])&&*chp2=='-';)
 	   for(;;)				       /* processing options */
 	    { switch(*++chp2)
-	       { case VERSIONOPT:elog(VERSION);
+	       { case VERSIONOPT:elog(procmailn);elog(VERSION);
 		    elog("\nLocking strategies:\tdotlocking");
 #ifndef NOfcntl_lock
 		    elog(", fcntl()");		    /* a peek under the hood */
@@ -97,7 +112,8 @@ main(argc,argv)const char*const argv[];
 		    elog(", flock()");
 #endif
 		    elog("\nDefault rcfile:\t\t");elog(pmrc);
-		    elog("\nSystem mailbox:\t\t");elog(systm_mbox);
+		    elog("\nYour system mailbox:\t");
+		    elog(auth_mailboxname(auth_finduid(getuid(),0)));
 		    elog(newline);
 		    return EXIT_SUCCESS;
 		 case HELPOPT1:case HELPOPT2:elog(pmusage);elog(PM_HELP);
@@ -184,22 +200,28 @@ conflopt:  nlog(conflicting),elog("options\n"),elog(pmusage);
       }
 #ifdef LD_ENV_FIX
      ;{ const char**emax=(const char**)environ,**ep;
-	static const char ld_[]="LD_";
+	static const char*ld_[]=
+	 {"LD_","_RLD","LIBPATH=","ELF_LD_","AOUT_LD_",0};
 	for(ep=emax;*emax;emax++);	  /* find the end of the environment */
-	while(*ep)
-	   if(!strncmp(ld_,*ep++,STRLEN(ld_)))	       /* it starts with LD_ */
-	      *--ep= *--emax,*emax=0;			/* copy from the end */
+	for(;*ep;ep++)
+	 { const char**ldp,*p;
+	   for(ldp=ld_;p= *ldp++;)
+	      if(!strncmp(*ep,p,strlen(p)))	/* if it starts with LD_ (or */
+	       { *ep--= *--emax;*emax=0;       /* similar) copy from the end */
+		 break;
+	       }
+	 }
       }
 #endif /* LD_ENV_FIX */
-     ;{ const struct passwd*pass,*passinvk;struct passwd spassinvk;int privs;
+     ;{ auth_identity*pass,*passinvk;auth_identity*spassinvk;int privs;
 	uid_t euid=geteuid();
-	spassinvk.pw_name=spassinvk.pw_dir=spassinvk.pw_shell=0;
-	passinvk=savepass(&spassinvk,uid=getuid());privs=1;gid=getgid();
+	spassinvk=auth_newid();passinvk=savepass(spassinvk,uid=getuid());
+	privs=1;gid=getgid();
 	;{ static const char*const trusted_ids[]=TRUSTED_IDS;
 	   if(Deliverymode&&*trusted_ids&&uid!=euid)
 	    { struct group*grp;const char*const*kp;
 	      if(passinvk)		      /* check out the invoker's uid */
-		 for(chp2=passinvk->pw_name,kp=trusted_ids;*kp;)
+		 for(chp2=(char*)auth_username(passinvk),kp=trusted_ids;*kp;)
 		    if(!strcmp(chp2,*kp++)) /* is it amongst the privileged? */
 		       goto privileged;
 	      if(grp=getgrgid(gid))	      /* check out the invoker's gid */
@@ -210,16 +232,23 @@ conflopt:  nlog(conflicting),elog("options\n"),elog(pmusage);
 	    }
 	 }
 privileged:				       /* move stdout out of the way */
-	endgrent();doumask(INIT_UMASK);savstdout=rdup(STDOUT);
+	endgrent();doumask(INIT_UMASK);
+	while((savstdout=rdup(STDOUT))<=STDERR)
+	 { rclose(savstdout);
+	   if(0>(savstdout=opena(devnull)))
+	      goto nodevnull;
+	   syslog(LOG_EMERG,"Descriptor %d was not open\n",savstdout);
+	 }
 	fclose(stdout);rclose(STDOUT);			/* just to make sure */
 	if(0>opena(devnull))
+nodevnull:
 	 { writeerr(devnull);syslog(LOG_EMERG,slogstr,errwwriting,devnull);
 	   return EX_OSFILE;			     /* couldn't open stdout */
 	 }
 #ifdef console
 	opnlog(console);
 #endif
-	setbuf(stdin,(char*)0);buf=malloc(linebuf);buf2=malloc(linebuf);
+	setbuf(stdin,(char*)0);mallocbuffers(linebuf);
 #ifdef SIGXCPU
 	signal(SIGXCPU,SIG_IGN);signal(SIGXFSZ,SIG_IGN);
 #endif
@@ -230,12 +259,16 @@ privileged:				       /* move stdout out of the way */
 	verboff();verbon();
 #else
 	verbon();verboff();
+#endif
+#ifdef SIGCHLD
+	signal(SIGCHLD,SIG_DFL);
 #endif
 	signal(SIGPIPE,SIG_IGN);qsignal(SIGTERM,srequeue);
 	qsignal(SIGINT,sbounce);qsignal(SIGHUP,sbounce);
 	qsignal(SIGQUIT,slose);signal(SIGALRM,(void(*)())ftimeout);
-	ultstr(0,(unsigned long)uid,buf);
-	chp2=!passinvk||!*passinvk->pw_name?buf:passinvk->pw_name;filled=0;
+	ultstr(0,(unsigned long)uid,buf);filled=0;
+	if(!passinvk||!(chp2=(char*)auth_username(passinvk)))
+	   chp2=buf;
 	;{ const char*fwhom;size_t lfr,linv;int tstamp;
 	   tstamp=fromwhom&&*fromwhom==REFRESH_TIME&&!fromwhom[1];fwhom=chp2;
 	   if(fromwhom&&!tstamp)
@@ -243,8 +276,8 @@ privileged:				       /* move stdout out of the way */
 		 privs=1; /* if -f user is the same as the invoker, allow it */
 	      if(!privs&&fromwhom&&override)
 	       { if(verbose)
-		    nlog(insufprivs);
-		 fromwhom=0;			      /* ignore the bogus -f */
+		    nlog(insufprivs);		      /* ignore the bogus -f */
+		 syslog(LOG_ERR,slogstr,attemptst,fwhom);fromwhom=0;
 	       }
 	      else
 		 fwhom=fromwhom;
@@ -283,6 +316,7 @@ privileged:				       /* move stdout out of the way */
 		    if(Deliverymode&&override&&!privs)
 		     { if(verbose)		  /* discard the bogus From_ */
 			  nlog(insufprivs);
+		       syslog(LOG_ERR,slogstr,attemptst,fwhom);
 		       goto no_from;
 		     }
 		    if(tstamp)
@@ -340,18 +374,13 @@ no_from:       { tstamp=0;	   /* no existing From_, so nothing to stamp */
 	   while(chp=(char*)argv[argc]);
 	gargv=argv+argc;suppmunreadable=verbose; /* save it for nextrcfile() */
 	if(Deliverymode)	/* try recipient without changing case first */
-	 { if(!(pass=getpwnam(chp2)))  /* chp2 should point to the recipient */
-	    { for(chp=chp2;*chp;chp++)	  /* kludge recipient into lowercase */
-		 if((unsigned)*chp-'A'<='Z'-'A')
-		    *chp+='a'-'A';	 /* getpwnam might be case sensitive */
-	      if(!(pass=getpwnam(chp2)))			/* try again */
-	       { static const char unkuser[]="Unknown user";
-		 nlog(unkuser);logqnl(chp2);
-		 syslog(LOG_ERR,slogstr,unkuser,chp2);
-		 return EX_NOUSER;		/* we don't handle strangers */
-	       }
+	 { if(!(pass=auth_finduser(chp2,-1)))	    /* chp2 is the recipient */
+	    { static const char unkuser[]="Unknown user";
+	      nlog(unkuser);logqnl(chp2);syslog(LOG_ERR,slogstr,unkuser,chp2);
+	      return EX_NOUSER;			/* we don't handle strangers */
 	    }
-	   if(enoughprivs(passinvk,euid,egid,pass->pw_uid,pass->pw_gid))
+	   if(enoughprivs(passinvk,euid,egid,auth_whatuid(pass),
+	    auth_whatgid(pass)))
 	      goto Setuser;
 	   nlog(insufprivs);
 	   syslog(LOG_CRIT,"Insufficient privileges to deliver to \"%s\"\n",
@@ -407,7 +436,7 @@ no_from:       { tstamp=0;	   /* no existing From_, so nothing to stamp */
 		       !enoughprivs(passinvk,euid,egid,stbuf.st_uid,
 			stbuf.st_gid)||		   /* can we do this at all? */
 		       S_ISDIR(stbuf.st_mode)||		  /* no directories! */
-		       !savepass(&spassinvk,stbuf.st_uid)    /* user exists? */
+		       !savepass(spassinvk,stbuf.st_uid)     /* user exists? */
 		      )
 nospecial:	     { static const char densppr[]=
 			"Denying special privileges for";
@@ -415,53 +444,64 @@ nospecial:	     { static const char densppr[]=
 		       syslog(LOG_ALERT,slogstr,densppr,rcfile);
 		     }
 		    else			    /* no security violation */
-		       mailfilter=2,passinvk= &spassinvk;
+		       mailfilter=2,passinvk=spassinvk;
 		  }				      /* accept new identity */
 	       }
 #endif
 	    }
 	 }
-	if(idhint&&(pass=getpwnam(idhint))&&
-	    passinvk&&passinvk->pw_uid==pass->pw_uid||
+	if(idhint&&(pass=auth_finduser((char*)idhint,0))&&
+	    passinvk&&auth_whatuid(passinvk)==auth_whatuid(pass)||
 	   (pass=passinvk))
 	  /*
 	   *	set preferred uid to the intended recipient
 	   */
-Setuser: { gid=pass->pw_gid;uid=pass->pw_uid;
-	   setdef(lgname,chp= *pass->pw_name?pass->pw_name:buf);
-	   setdef(home,pass->pw_dir);
+Setuser: { gid=auth_whatgid(pass);uid=auth_whatuid(pass);
+	   if(!*(chp=(char*)auth_username(pass)))
+	      chp=buf;
+	   setdef(lgname,chp);setdef(home,auth_homedir(pass));
 	   if(euid==ROOT_uid)
 	      initgroups(chp,gid);
-	   endgrent();setdef(shell,*pass->pw_shell?pass->pw_shell:binsh);
+	   endgrent();
+	   if(!*(chp=(char*)auth_shell(pass)))
+	      chp=(char*)binsh;
+	   setdef(shell,chp);setdef(orgmail,auth_mailboxname(pass));
 	 }
 	else		 /* user could not be found, set reasonable defaults */
 	  /*
 	   *	to prevent security holes, drop any privileges now
 	   */
 	 { setdef(lgname,buf);setdef(home,RootDir);setdef(shell,binsh);
-	   setids();
-	 }
-	endpwent();
-	if(passinvk)
-	 { free(spassinvk.pw_name);free(spassinvk.pw_dir);
-	   free(spassinvk.pw_shell);
+	   setdef(orgmail,"/tmp/dead.letter");setids();
 	 }
+	endpwent();auth_freeid(spassinvk);
       }
-     setdef(orgmail,systm_mbox);
      if(!presenviron||!mailfilter)	  /* by default override environment */
       { setdef(host,hostname());sputenv(lastfolder);sputenv(exitcode);
 	initdefenv();
 	;{ const char*const*kp;static const char*const prestenv[]=PRESTENV;
 	   for(kp=prestenv;*kp;)	/* preset some environment variables */
-	    { strcpy((char*)(sgetcp=buf2),*kp++);readparse(buf,sgetc,2);
-	      sputenv(buf);
+	    { strcpy((char*)(sgetcp=buf2),*kp++);
+	      if(readparse(buf,sgetc,2))
+		 setoverflow();
+	      else
+		 sputenv(buf);
 	    }
 	 }
       }		/* set fdefault and find out the name of our system lockfile */
-     sgetcp=fdefault;readparse(buf,sgetc,2);fdefault=tstrdup(buf);
+     sgetcp=fdefault;
+     if(readparse(buf,sgetc,2))		   /* uh, Houston, we have a problem */
+      { nlog(orgmail);elog(pathtoolong);elog(newline);
+	syslog(LOG_CRIT,"%s%s for LINEBUF for uid \"%lu\"\n",orgmail,
+	 pathtoolong,(unsigned long)uid);
+	fdefault="";
+	goto nix_sysmbox;
+      }
+     fdefault=tstrdup(buf);
      strcpy(chp2=strchr(strcpy(buf,chp=(char*)getenv(orgmail)),'\0'),lockext);
      defdeflock=tstrdup(buf);sgid=egid;accspooldir=3;	/* presumed innocent */
      if(mailfilter||!screenmailbox(chp,chp2,egid,Deliverymode))
+nix_sysmbox:
       { rcst_nosgid();sputenv(orgmail);	 /* nix delivering to system mailbox */
 	if(!strcmp(chp,fdefault))			/* DEFAULT the same? */
 	   free((char*)fdefault),fdefault="";			 /* so panic */
@@ -509,24 +549,50 @@ fake_rc:	 readerr(buf);
 	       { skiprc=0;
 		 goto nomore_rc;
 	       }
-	      suppmunreadable=0;
-findrc:	      i=0;		    /* should we keep the current directory? */
-	      if(rcfile==pmrc)		    /* the default .procmailrc file? */
-		 strcpy((char*)(rcfile=buf2),pmrc2buf());
-	      if(strchr(dirsep,*rcfile)||		   /* absolute path? */
+	      suppmunreadable=0;	      /* keep the current directory, */
+findrc:	      i=0;			      /* default rcfile, or neither? */
+	      if(rcfile==pmrc&&(i=2))	    /* the default .procmailrc file? */
+	       { if(*strcpy((char*)(rcfile=buf2),pmrc2buf())=='\0')
+pm_overflow:	  { strcpy(buf,pmrc);
+		    goto fake_rc;
+		  }
+	       }
+	      else if(strchr(dirsep,*rcfile)||		   /* absolute path? */
 		 (mailfilter||*rcfile==chCURDIR&&strchr(dirsep,rcfile[1]))&&
 		 (i=1))				     /* mailfilter or ./ pfx */
-		 *buf='\0';		/* do not put anything in front then */
+		 strcpy(buf,rcfile);	/* do not put anything in front then */
 	      else		     /* prepend default procmailrc directory */
-		 *lastdirsep(pmrc2buf())='\0';
-	      strcat(buf,rcfile);			/* append the rcfile */
-	      if(mailfilter!=2&&			 /* nothing special? */
-		 (stat(buf,&stbuf)?			      /* accessible? */
-		  rcstate==rc_NOSGID:stbuf.st_mode&S_IRUSR))	/* readable? */
-		 setids();				/* then transmogrify */
+	       { *(chp=lastdirsep(pmrc2buf()))='\0';
+		 if(buf[0]=='\0')
+		    goto pm_overflow;		  /* overflow in pmrc2buf()? */
+		 else
+		    strcpy(chp,rcfile);			/* append the rcfile */
+	       }
+	      if(mailfilter!=2&&(rcstate==rc_NOSGID||	 /* nothing special? */
+		  !rcstate&&!stat(buf,&stbuf)))	  /* don't need privilege or */
+		 setids();		  /* it's accessible?  Transmogrify! */
 	    }
 	   while(0>bopen(buf));			   /* try opening the rcfile */
-	   if(i)		  /* opened rcfile in the current directory? */
+	   if(rcstate!=rc_NORMAL&&mailfilter!=2)      /* if it isn't special */
+	    { closerc();			    /* but we are, close it, */
+	      setids();			  /* transmogrify to prevent peeking */
+	      if(0>bopen(buf))				    /* and try again */
+		 goto fake_rc;
+	    }
+#ifndef NOfstat
+	   if(fstat(rc,&stbuf))				    /* the right way */
+#else
+	   if(stat(buf,&stbuf))				  /* the best we can */
+#endif
+	    { static const char susprcf[]="Suspicious rcfile";
+susp_rc:      closerc();nlog(susprcf);logqnl(buf);
+	      syslog(LOG_ALERT,slogstr,susprcf,buf);
+	      goto fake_rc;
+	    }
+	   if(mailfilter==2)		 /* change now if we haven't already */
+	      setids();
+	   erestrict=1;			      /* possibly restrict execs now */
+	   if(i==1)		  /* opened rcfile in the current directory? */
 	    { if(!didchd)
 		 setmaildir(curdir);
 	    }
@@ -535,32 +601,41 @@ findrc:	      i=0;		    /* should we keep the current directory? */
 	      * OK, so now we have opened an absolute rcfile, but for security
 	      * reasons we only accept it if it is owned by the recipient or
 	      * root and is not world writable, and the directory it is in is
-	      * not world writable or has the sticky bit set
+	      * not world writable or has the sticky bit set.  If this is the
+	      * default rcfile then we also outlaw group writibility.
 	      */
-	    { i= *(chp=lastdirsep(buf));
-	      if(lstat(buf,&stbuf)||
-		 ((stbuf.st_uid!=uid&&stbuf.st_uid!=ROOT_uid||
-		  stbuf.st_mode&S_IWOTH)&&
-		  strcmp(devnull,buf)&&	      /* /dev/null is a special case */
-		  (*chp='\0',stat(buf,&stbuf)||
-		   (stbuf.st_mode&(S_IWOTH|S_IXOTH))==(S_IWOTH|S_IXOTH)&&
-		    !(stbuf.st_mode&S_ISVTX))))
-	       { static const char susprcf[]="Suspicious rcfile";
-		 *chp=i;closerc();nlog(susprcf);logqnl(buf);
-		 syslog(LOG_ALERT,slogstr,susprcf,buf);
-		 goto fake_rc;
+	    { register char c= *(chp=lastdirsep(buf));
+	      if(((stbuf.st_uid!=uid&&stbuf.st_uid!=ROOT_uid||
+		   stbuf.st_mode&S_IWOTH||
+		   i&&stbuf.st_mode&S_IWGRP&&(NO_CHECK_stgid||stbuf.st_gid!=gid)
+		  )&&strcmp(devnull,buf)||    /* /dev/null is a special case */
+		 (*chp='\0',stat(buf,&stbuf))||
+#ifdef CAN_chown
+		 !(stbuf.st_mode&S_ISVTX)&&
+#endif
+		 ((stbuf.st_mode&(S_IWOTH|S_IXOTH))==(S_IWOTH|S_IXOTH)||
+		  i&&(stbuf.st_mode&(S_IWGRP|S_IXGRP))==(S_IWGRP|S_IXGRP)
+		   &&(NO_CHECK_stgid||stbuf.st_gid!=gid))))
+	       { *chp=c;
+		 goto susp_rc;
 	       }
-	      *chp=i;
+	      *chp=c;
 	    }
 	  /*
 	   *	set uid back to recipient in any case, since we might just
 	   *	have opened his/her .procmailrc (don't remove these, since
 	   *	the rcfile might have been created after the first stat)
 	   */
-	   yell(drcfile,buf);setids();
+	   yell(drcfile,buf);
 	   if(!didchd)			       /* have we done this already? */
 	    { if((chp=lastdirsep(pmrc2buf()))>buf)	/* not the root dir? */
 		 chp[-1]='\0';		     /* eliminate trailing separator */
+	      if(buf[0]=='\0')					   /* arrrg! */
+	       { nlog("procmailrc");elog(pathtoolong);elog(newline);
+		 syslog(LOG_CRIT,"procmailrc%s for LINEBUF for uid \"%lu\"\n",
+		  pathtoolong,(unsigned long)uid);
+		 goto nomore_rc;		    /* just save it and pray */
+	       }
 	      if(chdir(chp=buf))      /* no, well, then try an initial chdir */
 	       { chderr(buf);
 		 if(chdir(chp=(char*)tgetenv(home)))
@@ -581,8 +656,10 @@ commint:   do skipspace();				  /* skip whitespace */
 	if(testB(':'))				       /* check for a recipe */
 	 { int locknext,succeed;char*startchar;long tobesent;
 	   static char flags[maxindex(exflags)];
-	   ;{ int nrcond;
-	      readparse(buf,getb,0);
+	   do
+	    { int nrcond;
+	      if(readparse(buf,getb,0))
+		 goto nextrc;			      /* give up on this one */
 	      ;{ char*chp3;
 		 nrcond=strtol(buf,&chp3,10);chp=chp3;
 	       }
@@ -612,11 +689,12 @@ commint:   do skipspace();				  /* skip whitespace */
 		 break;
 	       }			      /* parse & test the conditions */
 	      i=conditions(flags,prevcond,lastsucc,lastcond,nrcond);
+	      if(!flags[ALSO_NEXT_RECIPE]&&!flags[ALSO_N_IF_SUCC])
+		 lastcond=i==1;		   /* save the outcome for posterity */
+	      if(!prevcond||!flags[ELSE_DO])
+		 prevcond=i==1;	 /* same here, for `else if' like constructs */
 	    }
-	   if(!flags[ALSO_NEXT_RECIPE]&&!flags[ALSO_N_IF_SUCC])
-	      lastcond=i;		   /* save the outcome for posterity */
-	   if(!prevcond||!flags[ELSE_DO])
-	      prevcond=i;	 /* same here, for `else if' like constructs */
+	   while(i==2);			     /* missing in action, reiterate */
 	   startchar=themail;tobesent=filled;
 	   if(flags[PASS_HEAD])			    /* body, header or both? */
 	    { if(!flags[PASS_BODY])
@@ -635,7 +713,8 @@ progrm:	   if(testB('!'))				 /* forward the mail */
 	      chp=strchr(strcpy(buf,sendmail),'\0');
 	      if(*flagsendmail)
 		 chp=strchr(strcpy(chp+1,flagsendmail),'\0');
-	      readparse(chp+1,getb,0);
+	      if(readparse(chp+1,getb,0))
+		 goto fail;
 	      if(i)
 	       { if(startchar==themail)
 		  { startchar[filled]='\0';		     /* just in case */
@@ -649,7 +728,8 @@ progrm:	   if(testB('!'))				 /* forward the mail */
 	      skiprc--;
 	    }
 	   else if(testB('|'))				    /* pipe the mail */
-	    { getlline(buf2);			 /* get the command to start */
+	    { if(getlline(buf2))		 /* get the command to start */
+		 goto commint;
 	      if(i)
 	       { metaparse(buf2);
 		 if(!sh&&buf+1==Tmnate)		      /* just a pipe symbol? */
@@ -666,13 +746,16 @@ forward:	 if(locknext)
 			     buf2[i]='\0';
 			     if(sh)	 /* expand any environment variables */
 			      { chp=tstrdup(buf);sgetcp=buf2;
-				readparse(buf,sgetc,0);strcpy(buf2,buf);
-				strcpy(buf,chp);free(chp);
+				if(readparse(buf,sgetc,0))
+				 { *buf2='\0';
+				   goto nolock;
+				 }
+				strcpy(buf2,buf);strcpy(buf,chp);free(chp);
 			      }
 			     break;
 			   }
 		       if(!*buf2)
-			{ nlog("Couldn't determine implicit lockfile from");
+nolock:			{ nlog("Couldn't determine implicit lockfile from");
 			  logqnl(buf);
 			}
 		     }
@@ -693,20 +776,26 @@ forward:	 if(locknext)
 		  { if(!pipthrough(buf,startchar,tobesent))
 		       succeed=1,postStdout();	  /* only parse if no errors */
 		  }
-		 else if(!pipin(buf,startchar,tobesent)&& /* regular program */
-		  (succeed=1,!flags[CONTINUE]))
-		    goto frmailed;
+		 else if(!pipin(buf,startchar,tobesent))  /* regular program */
+		    if(succeed=1,flags[CONTINUE])
+		       goto logsetlsucc;
+		    else
+		       goto frmailed;
 		 goto setlsucc;
 	       }
 	    }
 	   else if(testB(EOF))
 	      nlog("Incomplete recipe\n");
 	   else		   /* dump the mail into a mailbox file or directory */
-	    { int ofiltflag;
+	    { int ofiltflag;char*end=buf+linebuf-4;	/* reserve some room */
 	      if(ofiltflag=flags[FILTER])
 		 flags[FILTER]=0,nlog(extrns),elog("filter-flag"),elog(ignrd);
-	      if(chp=gobenv(buf))	   /* can it be an environment name? */
-	       { if(skipspace())
+	      if(chp=gobenv(buf,end))	   /* can it be an environment name? */
+	       { if(chp==end)
+		  { getlline(buf);
+		    goto fail;
+		  }
+		 if(skipspace())
 		    chp++;		   /* keep pace with argument breaks */
 		 if(testB('='))		      /* is it really an assignment? */
 		  { int c;
@@ -771,7 +860,10 @@ forward:	 if(locknext)
 	       }
 	      if(!i)						/* no match? */
 		 skiprc++;		  /* temporarily disable subprograms */
-	      readparse(chp,getb,0);
+	      if(readparse(chp,getb,0))
+fail:	       { succeed=0;setoverflow();
+		 goto setlsucc;
+	       }
 	      if(i)
 	       { if(ofiltflag)	       /* protect who use bogus filter-flags */
 		    startchar=themail,tobesent=filled;	    /* whole message */
@@ -786,9 +878,9 @@ frmailed:	  { if(ifstack.offs)
 		       free(ifstack.offs);
 		    goto mailed;
 		  }
-setlsucc:	 if(succeed&&flags[CONTINUE]&&lgabstract==2)
+logsetlsucc:	 if(succeed&&flags[CONTINUE]&&lgabstract==2)
 		    logabstract(tgetenv(lastfolder));
-		 rawnonl=0;
+setlsucc:	 rawnonl=0;
 jsetlsucc:	 lastsucc=succeed;lasttell= -1;		       /* for comsat */
 	       }
 	      else
@@ -806,15 +898,26 @@ jsetlsucc:	 lastsucc=succeed;lasttell= -1;		       /* for comsat */
 	      skiprc--;					   /* decrease level */
 	 }
 	else				    /* then it must be an assignment */
-	 { if(!(chp=gobenv(buf)))
+	 { char*end=buf+linebuf;
+	   if(!(chp=gobenv(buf,end)))
 	    { if(!*buf)					/* skip a word first */
-		 getbl(buf);				      /* then a line */
+		 getbl(buf,buf+linebuf);		      /* then a line */
 	      skipped(buf);				/* display leftovers */
 	      continue;
 	    }
+	   if(chp==end)				      /* overflow => give up */
+nextrc:	      if(poprc()||wipetcrc())
+		 continue;
+	      else
+		 break;
 	   skipspace();
 	   if(testB('='))			   /* removal or assignment? */
-	      *chp='=',readparse(++chp,getb,1);
+	    { *chp='=';
+	      if(readparse(++chp,getb,1))
+	       { setoverflow();
+		 continue;
+	       }
+	    }
 	   else
 	      *++chp='\0';		     /* throw in a second terminator */
 	   if(!skiprc)
@@ -829,11 +932,15 @@ nomore_rc:
   ;{ int succeed;
      concon('\n');succeed=0;
      if(*(chp=(char*)fdefault))				     /* DEFAULT set? */
-      { setuid(uid);
+      { int len,tlen;
+	setuid(uid);
+	len=strlen(chp);		   /* make sure we have enough space */
+	if(linebuf<(tlen=len+strlen(lockext)+XTRAlinebuf+UNIQnamelen))
+	   mallocbuffers(linebuf=tlen);	   /* to perform the lock & delivery */
 	if(strcmp(chp,devnull)&&strcmp(chp,"|"))  /* neither /dev/null nor | */
 	 { cat(chp,lockext);
 	   if(!globlock||strcmp(buf,globlock))		  /* already locked? */
-	      lockit(buf,&loclock);			    /* implicit lock */
+	      lockit(tstrdup(buf),&loclock);		    /* implicit lock */
 	 }
 	if(writefolder(chp,(char*)0,themail,filled,0))		  /* default */
 	   succeed=1;
